
function [indices, gains, warning, nbre_mult] = ...
    algos_mp(F, x, K, algo, version)

% arguments algo et version
% 'MP'  : Basic MP, iterations until K different vectors found 
% 'MP+' : As above + gains optimization at the end (once K vectors found)
% 'OMP' : Orthogonal Matching Pursuit
% 'OOMP' ou 'MGS' : Optimized Orthogonal Matching Pursuit
% 'RMGS' : Recursive Modified Gram-Schmidt
% 'CMP'  : Complementary Matching Pursuit
% 'OCMP' : Orthogonal Complementary Matching Pursuit
% 'OOCMP' : Optimized Orthogonal Complementary Matching Pursuit
% 'CRMGS' : Complementary RMGS == OOCMP (but less complex)
% 'GP'   : Gradient Pursuit
% 'LAR'  : Least Angle Regression 
% 'LARS'  : Least Angle Regression - stepwise
% 'CyRMGS','CyOCMP','CyOOCMP' - cyclic RMGS 
%        with the initial subspace obtained with RMGS, OCMP, OOCMP, corresp.
%        version 'V1' : basic version
%        version 'V2' : simplified, less complex
% 'COSAMP' : alg. Needell and Tropp 
% 'SP' : Subspace Pursuit 
% 'SP_S': as above, version Sturm  (http://media.aau.dk/null_space_pursuits/2011/07/a-few-corrections-to-cosamp-and-sp-matlab.html)
% 'L1' : L1 minimization - basis pursuit
% 'MRMGS' : M-best RMGS see (P.Dymarski and R.Romaniuk 
%          "SPARSE SIGNAL APPROXIMATION ALGORITHMS IN A CELP CODER", EUSIPCO 2011)


[N, L] = size(F);
indices = ones(K,1);
gains = zeros(K,1);
warning = 0;
Eps = 1e-10;
Val_max = 1e+10;
arret_premature = 0;
switch algo
    case {'MP'} %%% Matching Pursuit %%%
        alpha = sum(F.^2);
        beta = x'*F;
        nbre_mult = 2*N*L;
        
        k = 0;
        Nbre_iter_mp=100; % max. number of iterations, but normal exit by 'break'

        for iter=1:Nbre_iter_mp

            [temp, indi] = max((beta.^2)./alpha);
            gaini = beta(indi)/alpha(indi);      % found vector of the index indi
            nbre_mult = nbre_mult+2*L;

            if iter == 1
                k = 1;
                indices(k)=indi;
                gains(k)=gaini;
                beta = beta - gains(k)*F(:,indices(k))'*F;
                nbre_mult = nbre_mult + N*L;
            else
            
                logi = 0;    
                for p = 1:k
                    if indi == indices(p)
                        logi = p;        % indi: repeating index
                    end
                end
                if logi == 0             % new index found
                    k = k+1;
                    indices(k) = indi;
                    gains(k)=gaini;
                    beta = beta - gains(k)*F(:,indices(k))'*F;
                    nbre_mult = nbre_mult + N*L;

                else                     % the old vector found once again
                    gains(logi)=gains(logi)+gaini;   % gain correction
                    beta = beta - gaini*F(:,indi)'*F;
                    nbre_mult = nbre_mult + N*L;
                end

             end
           
             if k == K
                break
             end
        end

        if k < K  % random indexes k...K and gains =0
            indi=2;
            for kk=1:100
                logi=0;
                for p = 1:k
                    if indi == indices(p)
                        logi=1;
                    end
                end
                if logi==0
                    k=k+1;
                    indices(k)=indi;
                    gains(k)=0.;
                else
                    indi=indi+1;
                end
                if k == K
                    break
                end
            end
        end
        
    case {'MP+'} %%% As above + gains optimization at the end (once K vectors found)
 
        alpha = sum(F.^2);
        beta = x'*F;
        nbre_mult = 2*N*L;
        
        k = 0;
        Nbre_iter_mp=100; % max. number of iterations, but normal exit by 'break'

        for iter=1:Nbre_iter_mp

            [temp, indi] = max((beta.^2)./alpha);
            gaini = beta(indi)/alpha(indi);      % found vector of the index indi
            nbre_mult = nbre_mult+2*L;


            if iter == 1
                k = 1;
                indices(k)=indi;
                gains(k)=gaini;
                beta = beta - gains(k)*F(:,indices(k))'*F;
                nbre_mult = nbre_mult + N*L;
            else
            
                logi = 0;    
                for p = 1:k
                    if indi == indices(p)
                        logi = p;        % indi: repeating index
                    end
                end
                if logi == 0             % new index found
                    k = k+1;
                    indices(k) = indi;
                    gains(k)=gaini;
                    beta = beta - gains(k)*F(:,indices(k))'*F;
                    nbre_mult = nbre_mult + N*L;

                else                     % the old vector found once again
                    gains(logi)=gains(logi)+gaini;   % gain correction
                    beta = beta - gaini*F(:,indi)'*F;
                    nbre_mult = nbre_mult + N*L;
                end

             end
           
             if k == K
                break
             end
        end       

        if k < K  % random indexes k...K
            indi=2;
            for kk=1:100
                logi=0;
                for p = 1:k
                    if indi == indices(p)
                        logi=1;
                    end
                end
                if logi==0
                    k=k+1;
                    indices(k)=indi;
                else
                    indi=indi+1;
                end
                if k == K
                    break
                end
            end
        end
        
        
        A = F(:,indices);
        gains = inv(A'*A)*A'*x;
        nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        
    case 'OMP' %%% Orthogonal Matching Pursuit %%%
        alpha = sum(F.^2);
        beta = x'*F;
        nbre_mult = 2*N*L;
        A = zeros(N,K);
        B = zeros(K,K);
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            A(:,k) = F(:,indices(k));
            if k == 1
                B(1,1) = A(:,1)'*A(:,1);
            else
                B(1:k,k) = A(:,1:k)'*A(:,k);
                B(k,1:k-1) = B(1:k-1,k);
            end
            nbre_mult = nbre_mult + N*k;
            gains(1:k) = inv(B(1:k,1:k))*A(:,1:k)'*x;
            nbre_mult = nbre_mult + k*(k*k/3 + N + k);
            if k < K
                e = x - F(:,indices(1:k))*gains(1:k);
                beta = e'*F;
                nbre_mult = nbre_mult + N*(L+k);
                alpha(indices(1:k)) = 1;
                beta(indices(1:k)) = 0;
            end
        end
        
    case {'OOMP','MGS'} %%% Optimized Orthogonal Matching Pursuit (or mgs) %%%
        alpha = sum(F.^2);
        beta = x'*F;
        nbre_mult = 2*N*L;
        F_orth = F;
        for k = 1:K
            [temp indices(k)] = max((beta.^2)./alpha);
            if k < K
                q = F_orth(:,indices(k))/sqrt(alpha(indices(k)));
                F_orth = F_orth - q*q'*F_orth;
                nbre_mult = nbre_mult + 2*N*L;
                alpha = sum(F_orth.^2);
                beta = x'*F_orth;
                nbre_mult = nbre_mult + 2*N*L;
                alpha(indices(1:k)) = 1;
                beta(indices(1:k)) = 0;
            end
        end
        A = F(:,indices);
        gains = inv(A'*A)*A'*x;
        nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        
    case 'RMGS'
        alpha = sum(F.^2);
        beta = x'*F;
        nbre_mult = 2*N*L;
        sqrt_alpha = zeros(K,1);
        gains_tilde = zeros(K,1);
        r = zeros(K,L);
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            sqrt_alpha(k) = sqrt(alpha(indices(k)));
            gains_tilde(k) = beta(indices(k))/sqrt_alpha(k);
            if k < K
                temp = r(1:k-1,indices(k))'*r(1:k-1,:);
                r(k,:) = F(:,indices(k))'*F - temp;
                r(k,:) = r(k,:)/sqrt_alpha(k);
                nbre_mult = nbre_mult + L*(N+k-1);
                alpha = alpha - r(k,:).^2;
                beta = beta - gains_tilde(k)*r(k,:);
                nbre_mult = nbre_mult + 2*L;
                alpha(indices(1:k)) = ones(1,k);
                beta(indices(1:k)) = zeros(1,k);
            end
        end
        gains(K) = gains_tilde(K)/sqrt_alpha(K);
        for k = K-1:-1:1
            temp = r(k,indices(k+1:K))*gains(k+1:K);
            gains(k) = (gains_tilde(k) - temp)/sqrt_alpha(k);
        end
        
    case 'CMP' %%% Complementary Matching Pursuit %%%
        B = inv(F*F');
        C = B*F;
        nbre_mult = N*N*N/3 + 2*N*N*L;
        alpha = sum(C.*F);
        beta = x'*C;
        nbre_mult = nbre_mult + 2*N*L;

        k = 0;
        Nbre_iter_mp=100; % max. number of iterations, but normal exit by 'break'

        for iter=1:Nbre_iter_mp

            [temp, indi] = max((beta.^2)./alpha);
            gaini = beta(indi)/alpha(indi);      % found vector of the index indi
            nbre_mult = nbre_mult+2*L;

            if iter == 1
                k = 1;
                indices(k)=indi;
                gains(k)=gaini;
                beta = beta - gains(k)*F(:,indices(k))'*C;
                nbre_mult = nbre_mult + N*L;
            else

                logi = 0;    
                for p = 1:k
                    if indi == indices(p)
                        logi = p;        % indi: repeating index
                    end
                end
                if logi == 0             % new index found
                    k = k+1;
                    indices(k) = indi;
                    gains(k)=gaini;
                    beta = beta - gains(k)*F(:,indices(k))'*C;
                    nbre_mult = nbre_mult + N*L;

                else                     % the old vector found once again
                    gains(logi)=gains(logi)+gaini;   % gain correction
                    beta = beta - gaini*F(:,indi)'*C;
                    nbre_mult = nbre_mult + N*L;
                end

             end
           
             if k == K
                break
             end
        end       


        if k < K  % random indexes k...K
            indi=2;
            for kk=1:100
                logi=0;
                for p = 1:k
                    if indi == indices(p)
                        logi=1;
                    end
                end
                if logi==0
                    k=k+1;
                    indices(k)=indi;
                else
                    indi=indi+1;
                end
                if k == K
                    break
                end
            end
        end

        A = F(:,indices);
        gains = inv(A'*A)*A'*x;
        nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);




        
    case 'OCMP' %%%Orth Complementary Matching Pursuit %%%
        psinv = F'*inv(F*F');
        D = psinv*F;
        nbre_mult = N*N*N/3 + 2*N*N*L + L*L*N;
        % operations precalculees, on peut mettre zero
        alpha = sum(D.*D);
        nbre_mult = nbre_mult + L*L;
        e = psinv*x;
        nbre_mult = nbre_mult + L*N;
        beta = e'*D;
        nbre_mult = nbre_mult + L*L;
        base = zeros(L,K);
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            if k < K
                q = D(:,indices(k))/sqrt(alpha(indices(k)));
                if k > 1
                    for i = 1:k-1
                        q = q - (q'*base(:,i))*base(:,i);
                        nbre_mult = nbre_mult + 2*L;
                    end
                    q = q/norm(q);
                end
                base(:,k) = q;
                
                % e est orthogonal a la base
                e = e - (e'*q)*q;
                nbre_mult = nbre_mult + 2*L;
                beta = e'*D;
                nbre_mult = nbre_mult + L*L;
                beta(indices(1:k)) = 0;
            end
        end
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'OOCMP' %%% Optimized Orth Complementary Matching Pursuit %%%
        B = inv(F*F');
        C = B*F;
        nbre_mult = N*N*N/3 + 2*N*N*L;
        alpha = sum(C.*F);
        beta = x'*C;
        nbre_mult = nbre_mult + 2*N*L;
        F_orth = C'*F;
        e = C'*x;
        nbre_mult = nbre_mult + N*L*L + N*L;
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            if k < K
                q = F_orth(:,indices(k))/sqrt(alpha(indices(k)));
                F_orth = F_orth - q*q'*F_orth;
                nbre_mult = nbre_mult + L*L + L*L;
                alpha = sum(F_orth.^2);
                beta = e'*F_orth;
                nbre_mult = nbre_mult + 2*L*L;
                alpha(indices(1:k)) = 1;
                beta(indices(1:k)) = 0;
            end
        end
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'CRMGS' %%% Complementary RMGS %%%
        B = inv(F*F');
        C = B*F;
        nbre_mult = N*N*N/3 + 2*N*N*L;
        
        alpha = sum(C.*F);
        beta = x'*C;
        nbre_mult = nbre_mult + 2*N*L;
        
        sqrt_alpha = zeros(K,1);
        gains_tilde = zeros(K,1);
        r = zeros(K,L);
        
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            sqrt_alpha(k) = sqrt(alpha(indices(k)));
            gains_tilde(k) = beta(indices(k))/sqrt_alpha(k);
            if k < K
                temp = r(1:k-1,indices(k))'*r(1:k-1,:);
                r(k,:) = F(:,indices(k))'*C - temp;
                r(k,:) = r(k,:)/sqrt_alpha(k);
                nbre_mult = nbre_mult + L*(N+k-1);
                alpha = alpha - r(k,:).^2;
                beta = beta - gains_tilde(k)*r(k,:);
                nbre_mult = nbre_mult + 2*L;
                alpha(indices(1:k)) = ones(1,k);
                beta(indices(1:k)) = zeros(1,k);
            end
        end
        
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'GP' %%% Gradient Pursuit %%%
        alpha = sum(F.^2);
        beta = x'*F;
        nbre_mult = 2*N*L;
        q = zeros(K,K);
        e = x;
        A = zeros(N,K);
        B = zeros(K,K);
        for k = 1:K
            [temp, indices(k)] = max((beta.^2)./alpha);
            A(:,k)= F(:,indices(k));
            e_tilde = A(:,1:k)'*e;
            nbre_mult = nbre_mult + N*k;
            q(1:k,k) = e_tilde;
            if k == 1
                B(1,1) = A(:,1)'*A(:,1);
                nbre_mult = nbre_mult + N*k;
            else
                B(1:k,k) = A(:,1:k)'*A(:,k);
                B(k,1:k-1) = B(1:k-1,k);
                nbre_mult = nbre_mult + N*k;
                for i = 1:k-1
                    temp = B(1:k,1:k)*q(1:k,i);
                    den = q(1:k,i)'*temp;
                    if abs(den) < Eps
                        % fprintf('abs(den) < eps k/K = %d/%d \n', k, K)
                        warning = 1;
                        return
                    end
                    q(1:k,k) = q(1:k,k) - (e_tilde'*temp)/den*q(1:k,i);
                    nbre_mult = nbre_mult + k*k + 2*k;
                end
            end
            den = q(1:k,k)'*B(1:k,1:k)*q(1:k,k);
            nbre_mult = nbre_mult + k*k + k;
            if abs(den) < Eps
                % fprintf('abs(den) < eps k/K = %d/%d \n', k, K)
                warning = 1;
                break
            end
            c = (e_tilde'*q(1:k,k))/den;
            gains(1:k) = gains(1:k) + c*q(1:k,k);
            e = x - A(:,1:k)*gains(1:k);
            nbre_mult = nbre_mult + k + k*N;
            if k < K
                beta = e'*F;
                nbre_mult = nbre_mult + N*L;
            end
        end
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'LAR' %%% Least Angle Regression %%%
        F_norm = F;
        for m = 1:L
            F_norm(:,m) = F_norm(:,m)/norm(F_norm(:,m));
        end
        beta = F_norm'*x;
        nbre_mult = 2*N*L;
        e = x;
        A = zeros(N,K);
        B = zeros(K,K);
        q = zeros(N,K);
        for k = 1:K
            [temp, indices(k)] = max((beta.^2));
            if k > 1 && indices(k) ~= prochain_indice
                fprintf('Pb : ind = %d %d \n', indices(k), prochain_indice)
            end
            
            A(:,k) = sign(beta(indices(k)))*F_norm(:,indices(k));
            if k == 1
                B(1,1) = A(:,1)'*A(:,1);
            else
                B(1:k,k) = A(:,1:k)'*A(:,k);
                B(k,1:k-1) = B(1:k-1,k);
            end
            nbre_mult = nbre_mult + N*k;
            Bm1 = inv(B(1:k,1:k));
            nbre_mult = nbre_mult + k^3/3;
            c = 1/sqrt((ones(1,k)*Bm1*ones(k,1)));
            w = c*Bm1*ones(k,1);
            q(:,k) = A(:,1:k)*w;
            nbre_mult = nbre_mult + k*k + k + N*k;
            
            if k < K
                r = F_norm'*q(:,k);
                nbre_mult = nbre_mult + N*L;
                beta_max = abs(beta(indices(k)));
                g1 = (beta_max-beta)./(c-r);
                g2 = (beta_max+beta)./(c+r);
                g1(find(g1 < 0)) = Val_max;
                g1(indices(1:k)) = Val_max;
                g2(find(g2 < 0)) = Val_max;
                g2(indices(1:k)) = Val_max;
                [g1_min j1_min] = min(g1);
                [g2_min j2_min] = min(g2);
                if g1_min < g2_min
                    g_min = g1_min;
                    prochain_indice = j1_min;
                else
                    g_min = g2_min;
                    prochain_indice = j2_min;
                end
                beta = beta - g_min*r;
                beta(indices(1:k)) = zeros(k,1);
            else
                g_min = q(:,k)'*e;
                nbre_mult = nbre_mult + N;
            end
            e = e - g_min*q(:,k);
            if norm(e) < Eps
                arret_premature = 1;
                break
            end
        end
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'LARS' %%% Least Angle Regression - stepwise %%%
        Nbre_iter_lar=200; % max number of iterations - typical exit by 'break'
        DD = F;
        for m = 1:L
            DD(:,m) = DD(:,m)/norm(DD(:,m));
        end
        e = x;
        indices = ones(K,1);
        step = 10*norm(e)/Nbre_iter_lar;  % "step" influences precision and complexity
        k = 0;
        nbre_mult = 0; 
        for iter=1:Nbre_iter_lar
            inter_cor = e'*DD;
            nbre_mult = nbre_mult + N*L; 
            [temp, indi] = max(inter_cor.*inter_cor);     % index
            sigi = inter_cor(indi)/abs(inter_cor(indi));  % sign

            if iter == 0
                k = 1;
                indices(k)=indi;
            else
            
                logi = 0;
                for p = 1:k
                    if indi == indices(p)
                        logi = 1;
                    end
                end
                if logi == 0    % new vector found
                    k = k+1;
                    indices(k) = indi;
                end
            end
            e = e - DD(:,indi)*step*sigi;  % 
            nbre_mult = nbre_mult + N;  
            
            if k == K
                break
            end
        end
        
        if k < K  % random indexes k...K
            indi=2;
            for kk=1:100
                logi=0;
                for p = 1:k
                    if indi == indices(p)
                        logi=1;
                    end
                end
                if logi==0
                    k=k+1;
                    indices(k)=indi;
                else
                    indi=indi+1;
                end
                if k == K
                    break
                end
            end
        end
        
        % Optimization of the gains
        % A = F(:,indices(1:k)');
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
        %      x_hat = F(:,indices(1:k)')*gains;
        
    case {'CyRMGS','CyOCMP','CyOOCMP'}
        alpha_init = sum(F.^2);
        beta_init = x'*F;
        % First stage: RMGS, OCMP or OOCMP
        switch algo
            case 'CyRMGS' %%% RMGS at the first stage %%%
                alpha = alpha_init;
                beta = beta_init;
                nbre_mult = 2*N*L; % le calcul de alpha_init et beta_init
                % ne doit pas tre comptabilis par OCMP+
                sqrt_alpha = zeros(K,1);
                gains_tilde = zeros(K,1);
                r = zeros(K,L);
                for k = 1:K
                    [temp, indices(k)] = max((beta.^2)./alpha);
                    sqrt_alpha(k) = sqrt(alpha(indices(k)));
                    gains_tilde(k) = beta(indices(k))/sqrt_alpha(k);
                    if k < K
                        temp = r(1:k-1,indices(k))'*r(1:k-1,:);
                        r(k,:) = F(:,indices(k))'*F - temp;
                        r(k,:) = r(k,:)/sqrt_alpha(k);
                        nbre_mult = nbre_mult + L*(N+k-1);
                        alpha = alpha - r(k,:).^2;
                        beta = beta - gains_tilde(k)*r(k,:);
                        nbre_mult = nbre_mult + 2*L;
                        alpha(indices(1:k)) = ones(1,k);
                        beta(indices(1:k)) = zeros(1,k);
                    end
                end
                gains(K) = gains_tilde(K)/sqrt_alpha(K);
                for k = K-1:-1:1
                    temp = r(k,indices(k+1:K))*gains(k+1:K);
                    gains(k) = (gains_tilde(k) - temp)/sqrt_alpha(k);
                end
            case 'CyOCMP' %%% OCMP at the first stage %%%
                psinv = F'*inv(F*F');
                D = psinv*F;
                nbre_mult = N*N*N/3 + 2*N*N*L + L*L*N;
                % operations precalculees, on peut mettre zero
                alpha = sum(D.*D);
                nbre_mult = nbre_mult + L*L;
                e = psinv*x;
                nbre_mult = nbre_mult + L*N;
                beta = e'*D;
                nbre_mult = nbre_mult + L*L;
                base = zeros(L,K);
                for k = 1:K
                    [temp, indices(k)] = max((beta.^2)./alpha);
                    if k < K
                        q = D(:,indices(k))/sqrt(alpha(indices(k)));
                        if k > 1
                            for i = 1:k-1
                                q = q - (q'*base(:,i))*base(:,i);
                                nbre_mult = nbre_mult + 2*L;
                            end
                            q = q/norm(q);
                        end
                        base(:,k) = q;
                        
                        % e est orthogonal a la base
                        e = e - (e'*q)*q;
                        nbre_mult = nbre_mult + 2*L;
                        beta = e'*D;
                        nbre_mult = nbre_mult + L*L;
                        beta(indices(1:k)) = 0;
                    end
                end
                A = F(:,indices);
                if rcond(A'*A) < Eps
                    warning = 1;
                    return
                end
                gains = inv(A'*A)*A'*x;
                nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
                
            case 'CyOOCMP' %%% OOCMP at the first stage %%%
                B = inv(F*F');
                C = B*F;
                nbre_mult = N*N*N/3 + 2*N*N*L;
                
                alpha = sum(C.*F);
                beta = x'*C;
                nbre_mult = nbre_mult + 2*N*L;
                
                sqrt_alpha = zeros(K,1);
                gains_tilde = zeros(K,1);
                r = zeros(K,L);
                
                for k = 1:K
                    [temp, indices(k)] = max((beta.^2)./alpha);
                    sqrt_alpha(k) = sqrt(alpha(indices(k)));
                    gains_tilde(k) = beta(indices(k))/sqrt_alpha(k);
                    if k < K
                        temp = r(1:k-1,indices(k))'*r(1:k-1,:);
                        r(k,:) = F(:,indices(k))'*C - temp;
                        r(k,:) = r(k,:)/sqrt_alpha(k);
                        nbre_mult = nbre_mult + L*(N+k-1);
                        alpha = alpha - r(k,:).^2;
                        beta = beta - gains_tilde(k)*r(k,:);
                        nbre_mult = nbre_mult + 2*L;
                        alpha(indices(1:k)) = ones(1,k);
                        beta(indices(1:k)) = zeros(1,k);
                    end
                end
                
                A = F(:,indices);
                if rcond(A'*A) < Eps
                    warning = 1;
                else
                    gains = inv(A'*A)*A'*x;
                    nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
                end
        end
        
        
        % Rejection of the unnecessary vectors, if the error equals (almost) zero
        if 0                                 % now suppressed
            x_hat = F(:,indices)*gains;
            err = (x - x_hat)'*(x - x_hat);
            if err < Eps
                k = 1;
                while k <= length(indices)
                    indices_new = [indices(1:k-1); indices(k+1:end)];
                    A = F(:,indices_new);
                    x_hat = A*inv(A'*A)*A'*x;
                    err = (x - x_hat)'*(x - x_hat);
                    if err < Eps
                        indices = indices_new;
                    else
                        k = k + 1;
                    end
                end
            end
            if length(indices) < K
                fprintf('alg 8 case 1 : K = %2d -> %2d\n', K, length(indices))
            end
        end
        
        % Second stage: Cyclic RMGS
        
        Nbre_iter_max = 10;
        A = F(:,indices);
        x_hat = A*gains;
        nbre_mult = nbre_mult + N*K;
        xnormax = norm(x_hat);
        
        for no_iter = 1:Nbre_iter_max
            indices_old = indices;
            switch version
                case 'V1' % basic version, more complex
                    if K == 1
                        [temp, j_opt] = max((beta_init.^2)./alpha_init);
                        indices_temp = j_opt;
                        gains_temp = beta_init(j_opt)/alpha_init(j_opt);
                        A = F(:,indices_temp);
                        xnorm = norm(A*gains_temp);
                        nbre_mult = nbre_mult + (K+1)*N;
                        if xnorm > xnormax
                            xnormax = xnorm;
                            indices = indices_temp;
                            gains = gains_temp;
                        end
                    else
                        for k = 1:K
                            ind = [indices(1:k-1); indices(k+1:K)];
                            alpha = alpha_init;
                            beta = beta_init;
                            r = zeros(K,L);
                            for p = 1:K-1
                                sqrt_alpha(p) = sqrt(alpha(ind(p)));
                                gains_tilde(p) = beta(ind(p))/sqrt_alpha(p);
                                temp = r(1:p-1,ind(p))'*r(1:p-1,:);
                                r(p,:) = F(:,ind(p))'*F - temp;
                                r(p,:) = r(p,:)/sqrt_alpha(p);
                                nbre_mult = nbre_mult + L*(N+p-1);
                                alpha = alpha - r(p,:).^2;
                                beta = beta - gains_tilde(p)*r(p,:);
                                nbre_mult = nbre_mult + 2*L;
                                alpha(ind(1:p)) = ones(1,p);
                                beta(ind(1:p)) = zeros(1,p);
                            end
                            [temp, j_opt] = max((beta.^2)./alpha);
                            sqrt_alpha(K) = sqrt(alpha(j_opt));
                            gains_tilde(K) = beta(j_opt)/sqrt_alpha(K);
                            indices_temp = [ind; j_opt];
                            gains_temp = zeros(K,1);
                            gains_temp(K) = gains_tilde(K)/sqrt_alpha(K);
                            for p = K-1:-1:1
                                temp = r(p,indices_temp(p+1:K))*gains_temp(p+1:K);
                                gains_temp(p) = (gains_tilde(p) - temp)/sqrt_alpha(p);
                            end
                            A = F(:,indices_temp);
                            xnorm = norm(A*gains_temp);
                            nbre_mult = nbre_mult + (K+1)*N;
                            if xnorm > xnormax
                                xnormax = xnorm;
                                indices = indices_temp;
                                gains = gains_temp;
                            end
                        end
                    end
                    
                case 'V2' % simplified version, orthogolalization less frequent
                    F_orth = F;
                    for k = 1:K
                        q = F_orth(:,indices(k));
                        q = q/norm(q);
                        nbre_mult = nbre_mult + N;
                        F_orth = F_orth - q*q'*F_orth;
                        nbre_mult = nbre_mult + 2*N*L;
                    end
                    alpha = sum(F_orth.^2);
                    nbre_mult = nbre_mult + N*L;
                    if K == 1
                        beta = x'*F_orth;
                        nbre_mult = nbre_mult + N*L;
                        [temp, j_opt] = max((beta.^2)./alpha);
                        indices_temp = j_opt;
                        gains_temp = beta(j_opt)/alpha(j_opt);
                        xnorm = norm(F(:,indices_temp)*gains_temp);
                        nbre_mult = nbre_mult + (K+1)*N;
                        if xnorm > xnormax
                            xnormax = xnorm;
                            indices = indices_temp;
                            gains = gains_temp;
                        end
                    else
                        for k = 1:K
                            ind = indices([(1:k-1), (k+1:K)]);
                            A = F(:,ind);
                            gains_temp = inv(A'*A)*A'*x;
                            nbre_mult = nbre_mult + ...
                                (K-1)*((K-1)*N + (K-1)^2/3 + N + (K-1));
                            x_hat = A*gains_temp;
                            nbre_mult = nbre_mult + N*(K-1);
                            e = x - x_hat;
                            beta = e'*F_orth;
                            nbre_mult = nbre_mult + N*L;
                            alpha(ind) = 1;
                            beta(ind) = 0;
                            [temp, j_opt] = max((beta.^2)./alpha);
                            indices_temp = [ind; j_opt];
                            A = F(:,indices_temp);
                            if rcond(A'*A) < Eps
                                warning = 1;
                                return
                            else
                                gains_temp = inv(A'*A)*A'*x;
                                nbre_mult = nbre_mult + ...
                                    K*(K*N + K^2/3 + N + K);
                            end
                            xnorm = norm(A*gains_temp);
                            nbre_mult = nbre_mult + (K+1)*N;
                            if xnorm > xnormax
                                xnormax = xnorm;
                                indices = indices_temp;
                                gains = gains_temp;
                            end
                        end
                    end                    
            end
            if sum(sort(indices_old) ~= sort(indices)) == 0
                % fprintf(' %s K = %d : indices_old == indices\n', algo, K)
                break
            end
        end
        
        
 % Rejection of the unnecessary vectors, if the error equals (almost) zero
        if 0                                 % now suppressed
        
            A = F(:,indices);
            x_hat = A*inv(A'*A)*A'*x;
            err = (x - x_hat)'*(x - x_hat);
            if err < Eps
                k = 1;
                while k <= length(indices)
                    indices_new = [indices(1:k-1); indices(k+1:end)];
                    A = F(:,indices_new');
                    x_hat = A*inv(A'*A)*A'*x;
                    err = (x - x_hat)'*(x - x_hat);
                    if err < Eps
                        indices = indices_new;
                    else
                        k = k + 1;
                    end
                end
            end
            if length(indices) < P
                fprintf('alg 8 case 2 : P = %2d -> %2d\n', P, length(indices))
            end
        end
        
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
            nbre_mult = nbre_mult + K*(K*N + K*K/3 + N + K);
        end
        
    case 'COSAMP' %%% COSAMP as described by Needell and Tropp, stopping cond. modified %%%
        DD = F;
        for m = 1:L
            DD(:,m) = DD(:,m)/norm(DD(:,m));
        end
        indices = zeros(K,1);
        %e = x;
        [lgains,ii] = cosamp_(DD,x,K,0.001,0.0001);
        k = 0;
        for i = 1:L
            if abs(lgains(i)) > 10^(-10)
                k = k+1;
                indices(k) = i;
            end
        end
        
        if k < K  % indices aleatoires k...K
            indi=2;
            for kk=1:100
                logi=0;
                for p = 1:k
                    if indi == indices(p)
                        logi=1;
                    end
                end
                if logi==0
                    k=k+1;
                    indices(k)=indi;
                else
                    indi=indi+1;
                end
                if k == K
                    break
                end
            end
        end
        
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
        end
        
        nbre_mult = ii*cout_inv(N,3*K);  % tres approximativement
        
    case 'SP' %%% algorithm Subspace Pursuit, stopping condition modified %%%
        DD=F;
        for m = 1:L
            DD(:,m) = DD(:,m)/norm(DD(:,m));
        end
        indices = zeros(K,1);
        %e=x;
        [lgains,ii]=sub_pursuit(DD,x,K,0.001,0.0001);
        k=0;
        for i=1:L
            if abs(lgains(i)) > 10^(-10)
                k=k+1;
                indices(k)=i;
            end
        end
        
        if k < K  % indices aleatoires k...K
            indi=2;
            for kk=1:100
                logi=0;
                for p = 1:k
                    if indi == indices(p)
                        logi=1;
                    end
                end
                if logi==0
                    k=k+1;
                    indices(k)=indi;
                else
                    indi=indi+1;
                end
                if k == K
                    break
                end
            end
        end
        
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
        end
               
        nbre_mult=ii*(cout_inv(N,2*K)+cout_inv(N,K));  % tres approximativement
        
    case 'SP_S' %%% algorithm Subspace Pursuit, version B.Sturm, stopping condition modified %%%
        DD=F;
        for m = 1:L
            DD(:,m) = DD(:,m)/norm(DD(:,m));
        end
        indices = zeros(K,1);
        %e=x;
        [lgains,ii]=sp_s(DD,x,K,0.001,0.0001);
        k=0;
        for i=1:L
            if abs(lgains(i)) > 10^(-10)
                k=k+1;
                indices(k)=i;
            end
        end
        
        if k < K  % indices aleatoires k...K
            indi=2;
            for kk=1:100
                logi=0;
                for p = 1:k
                    if indi == indices(p)
                        logi=1;
                    end
                end
                if logi==0
                    k=k+1;
                    indices(k)=indi;
                else
                    indi=indi+1;
                end
                if k == K
                    break
                end
            end
        end
        
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
        end
               
        nbre_mult=ii*(cout_inv(N,2*K)+cout_inv(N,K));  % tres approximativement


    case 'L1' %%% algorithm min L1 - basis pursuit %%%
        DD=F;
        for m = 1:L
            DD(:,m) = DD(:,m)/norm(DD(:,m));
        end
        indices = zeros(K,1);
        %e = x;
        ii = 0;
        lambda=1.;
        lb=log(5);
        la=log(0.01);
        if K > 2
            lambda=0.5;
            lb=log(2);
            la=log(0.01);
        end
        if K >= 4
            lambda=0.1;
            lb=log(1);
            la=log(0.001);
        end
        if K >= 6
            lambda=0.05;
            lb=log(0.1);
            la=log(0.0005);
        end
        if K >= 8
            lambda=0.01;
            lb=log(0.05);
            la=log(0.0001);
        end
        if K >= 10
            lambda=0.005;
            lb=log(0.02);
            la=log(0.0001);
        end
        if K > 12
            lambda=0.0005;
            lb=log(0.001);
            la=log(0.00001);
        end
        
        epsi=0.00001;
        maxiter=100;
        crr=ones(11,1);
        for ij=1:11
            ld=la+(ij-1)*(lb-la)/10;
            crd=bp(F,K,DD,x,L,maxiter,ld,epsi);
            crr(ij)=crd;
        end
        ii=ii+9;
        [err,iin]=min(crr);
        lc=la+(iin-1)*(lb-la)/10;
        lambda=exp(lc);
        lgains=SolveBP(DD,x,L,maxiter,lambda,epsi);
        ii=ii+1;
        kk=0;
        for i=1:L
            if abs(lgains(i)) > 10^(-6)
                kk=kk+1;
            end
        end
        
        [lgai,indii]=sort(abs(lgains),'descend');
        k=min(kk,K);
        lgainss=zeros(size(lgains));
        for i=1:k
            indices(i)=indii(i);
            lgainss(indii(i))=lgains(indii(i));
        end
        
        if k < K  % indices aleatoires k...K
            indi=2;
            for kk=1:100
                logi=0;
                for p = 1:k
                    if indi == indices(p)
                        logi=1;
                    end
                end
                if logi==0
                    k=k+1;
                    indices(k)=indi;
                else
                    indi=indi+1;
                end
                if k == K
                    break
                end
            end
        end
        
        A = F(:,indices);
        if rcond(A'*A) < Eps
            warning = 1;
        else
            gains = inv(A'*A)*A'*x;
        end
        
        nbre_mult = ii*(0.8*K+6.6)*10^6; %  ii * temps mesure pour 1 execution de solvebp

   case 'MRMGS'  %%%% M-best RMGS

        kb=10;  % number of parallel solutions

        indices = zeros(K,1);
        gains = zeros(K,1);

  if norm(x) > 0   % if signal not equal to 0
        
        mindices = zeros(K,kb);
        mgains = zeros(K,kb);
        msqrt_alpha = zeros(K,kb);
        mgains_tilde = zeros(K,kb);
        
        cum=zeros(1,kb); % cumulated projections
        
        LL=kb*L;
        mr = zeros(K,LL);
        
        alpha = sum(F.^2);
        beta = x'*F;
        
        malpha=zeros(1,LL);
        mbeta=zeros(1,LL);
        for i=1:kb
          mpocz=1+(i-1)*L;
          malpha(mpocz:mpocz+L-1)=alpha(1,:);
          mbeta(mpocz:mpocz+L-1)=beta(1,:);
          
        end
        
        sqrt_alpha = zeros(K,1);
        gains_tilde = zeros(K,1);
        r = zeros(K,L);
        
%  first iteration (k=1) - start of kb paths
           k=1;
           [absort, abindex] = sort((beta.^2)./alpha,'descend');
           indices1=abindex(1:kb);  % indexes of kb best vectors
           
           for i=1:kb
            msqrt_alpha(k,i) = sqrt(alpha(indices1(i)));
            mgains_tilde(k,i) = beta(indices1(i))/msqrt_alpha(k,i);
            cum(i)=absort(i);
           end
           
           if K == 1   % for 1 vector, the RMGS alg. is used
            [tempu, indices(k)] = max((beta.^2)./alpha);
            sqrt_alpha(k) = sqrt(alpha(indices(k)));
            gains_tilde(k) = beta(indices(k))/sqrt_alpha(k);
            cum(1)=tempu;
           else
             for i=1:kb
                mpocz=1+(i-1)*L;
                mindices(k,i)=indices1(i);
                mr(k,mpocz:mpocz+L-1) = F(:,mindices(k,i))'*F;
                mr(k,mpocz:mpocz+L-1) = mr(k,mpocz:mpocz+L-1)/msqrt_alpha(k,i);
                pst=zeros(1,L);
                pst(1,:)=mr(k,mpocz:mpocz+L-1);
                malpha(1,mpocz:mpocz+L-1) = malpha(1,mpocz:mpocz+L-1) - pst.^2; 
                pst(1,:)=mgains_tilde(k,i)*mr(k,mpocz:mpocz+L-1);
                mbeta(1,mpocz:mpocz+L-1) = mbeta(1,mpocz:mpocz+L-1) - pst; 
                
                malpha(1,mpocz-1+mindices(1:k,i)) = ones(1,k);
                mbeta(1,mpocz-1+mindices(1:k,i)) = zeros(1,k);
             end
           end
        
% next iterations
        for k = 2:K
   
   % searching for kb best vectors
   
             crite=zeros(1,LL);
             for i=1:kb
                 mpocz=1+(i-1)*L;
                 crite(1,mpocz:mpocz+L-1)=cum(i);
             end
             crite=crite+(mbeta.^2)./malpha;
             [mabsort, mabindex] = sort(crite,'descend');
             
             mabsort1=zeros(1,kb);
             mindices1=zeros(1,kb);
             mindices_comp=zeros(k,kb);
             indk=1;
             for i=1:LL
                ib=fix((mabindex(i)-1)/L);
                ind=mabindex(i)-ib*L;
                ibb=ib+1;
                mindices_comp(1:(k-1),indk)=mindices(1:(k-1),ibb);
                mindices_comp(k,indk)=ind;
                mindices_comp=sort(mindices_comp);
                
                if indk==1
                    mindices1(indk)=mabindex(1);
                    mabsort1(indk)=mabsort(1);
                    indk=2;
                else
                    wsk=1.;
                    for ii=1:indk-1
                        wsk=wsk*norm(mindices_comp(:,indk)-mindices_comp(:,ii));
                    end
                    if wsk > 10^(-10)
                        mindices1(indk)=mabindex(i);
                        mabsort1(indk)=mabsort(i);
                        indk=indk+1;
                    end
                    if indk == kb+1
                        break
                    end
                end
             end
                        
             
    %         mindices1=mabindex(1:kb);  % indexes of kb best vectors
      
            
   % copying of kb paths, appending the last result
      
            mindices_=mindices;
            malpha_=malpha;
            mbeta_=mbeta;
            msqrt_alpha_ =msqrt_alpha; 
            mgains_tilde_ =mgains_tilde;
            mr_=mr;
            
            for i=1:kb
                ib=fix((mindices1(i)-1)/L);
                ind=mindices1(i)-ib*L;
                ibb=ib+1;
         
                mpocz_=ib*L+1;
                mpocz=(i-1)*L+1;
                mindices(:,i)= mindices_(:,ibb);
                malpha(mpocz:mpocz+L-1)=malpha_(mpocz_:mpocz_+L-1);
                mbeta(mpocz:mpocz+L-1)=mbeta_(mpocz_:mpocz_+L-1);
                msqrt_alpha(:,i) =msqrt_alpha_(:,ibb);
                mgains_tilde(:,i)=mgains_tilde_(:,ibb);
                mr(:,mpocz:mpocz+L-1)=mr_(:,mpocz_:mpocz_+L-1);
                
                mindices(k,i)=ind;
                cum(i)=mabsort1(i); %%%%%%%%%%%%%%
                msqrt_alpha(k,i) = sqrt(malpha(mpocz-1+ind));
                mgains_tilde(k,i) = mbeta(mpocz-1+ind)/msqrt_alpha(k,i);
                
             
              if k < K
                  
                tempo = mr(1:k-1,mpocz-1+mindices(k,i))'*mr(1:k-1,mpocz:mpocz+L-1);   
                mr(k,mpocz:mpocz+L-1) = F(:,mindices(k,i))'*F - tempo;
                mr(k,mpocz:mpocz+L-1) = mr(k,mpocz:mpocz+L-1)/msqrt_alpha(k,i); 
                
                  
                
  
                pst=zeros(1,L);
                pst(1,:)=mr(k,mpocz:mpocz+L-1);
                malpha(mpocz:mpocz+L-1) = malpha(mpocz:mpocz+L-1) - pst.^2; 
                
                pst(1,:)=mgains_tilde(k,i)*mr(k,mpocz:mpocz+L-1);
                mbeta(mpocz:mpocz+L-1) = mbeta(mpocz:mpocz+L-1) - pst; 
                
                malpha(1,mpocz-1+mindices(1:k,i)) = ones(1,k);
                mbeta(1,mpocz-1+mindices(1:k,i)) = zeros(1,k);
                
              end
           end
        end
        
        if K == 1  
         gains(K) = gains_tilde(K)/sqrt_alpha(K);
        else  % choice of the best path
            errmax=10^6;
            iopt=0;
            erri=zeros(kb,1);
            for i=1:kb
                mgains(K,i) = mgains_tilde(K,i)/msqrt_alpha(K,i); 
                mpocz=(i-1)*L+1;
                for k = K-1:-1:1  
                  temp = mr(k,mpocz-1+mindices(k+1:K,i))*mgains(k+1:K,i);
                  
                  mgains(k,i) = (mgains_tilde(k,i) - temp)/msqrt_alpha(k,i);   
                    
                end 
               
                x_hat = F(:,mindices(:,i))*mgains(:,i);
                erri(i)=norm(x-x_hat);
                 if erri(i) < errmax
                     errmax=erri(i);
                     indices(:,1)=mindices(:,i);
                     gains(:,1)=mgains(:,i);
                     iopt=i;
                 end
            end
%             cum
%             erri
%             iopt
%             mindices'
            

        end    
        
  else    
    for m=1:K
        indices(m,1)=m;
        gains(m,1)=0;
    end
  end    

      
%        A = F(:,indices');
%        gains = inv(A'*A)*A'*x;
%        x_hat = F(:,indices)*gains;  % the signal model

       nbre_mult =kb*((K+1)*N*L + (K-1)*L*(1+K/2)); %% first approx., kb times cost of RMGS
end

% Vrification des formules suivantes except
% pour tous les algos si warning = 1
% pour l'algo LAR si arret_premature = 1
% pour les algos avec substitution
if ~warning
    a = sum(1:K);
    b = sum((1:K).^2);
    c = sum((1:K).^3);
    switch algo
        case 'MP',   cout = (K+1)*N*L;
        case 'MP+',  cout = (K+1)*N*L + cout_inv(N,K);
        case 'OMP',  cout = (K+1)*N*L + 3*N*sum(1:K-1) + 2*N*K + b + c/3;
        case 'OOMP', cout = (K+1)*N*L + 3*(K-1)*N*L + cout_inv(N,K);
        case 'RMGS', cout = (K+1)*N*L + (K-1)*L*(1+K/2);
        case 'MRMGS', cout = kb*((K+1)*N*L + (K-1)*L*(1+K/2));
        case 'CMP',  cout = (K+1)*N*L + 2*N*N*L + N^3/3 + cout_inv(N,K);
        case 'OCMP',
            cout = N*N*N/3 + 2*N*N*L + L*L*N;
            cout = cout + L*L + L*N + 2*sum(1:K-2)*L;
            cout = cout + K*(2*L + L*L) - 2*L;
            cout = cout + cout_inv(N,K);
        case 'OOCMP',
            cout = N*N*N/3 + 2*N*N*L;
            if 0
                % ancienne formule
                cout = cout + 3*N*L + N*L*L + 4*(K-1)*L*L;
                cout = cout + cout_inv(N,K);
            else
                % nouvelle formule non vrifie (janvier 2011)
                cout = cout + 4*K*N*L;
                cout = cout + cout_inv(N,K);
                nbre_mult = cout;
            end
            
        case 'CRMGS', cout=(K+1)*N*L + 2*N*N*L + N^3/3 + cout_inv(N,K)+ (K-1)*L*(1+K/2);
            
        case 'GP',   cout = (K+1)*N*L + 3*N*a + 2*b + c + cout_inv(N,K);
        case 'LAR',
            if ~arret_premature
                cout = (K+1)*N*L + (2*N+1)*a + b + c/3 + N + cout_inv(N,K);
            else
                cout = nbre_mult;
            end
        case {'LARS','COSAMP','SP','SP_S','L1'}
            cout = nbre_mult;
        case {'CyRMGS','CyOCMP','CyOOCMP'}
            if 0
                switch algo
                    case 'RMGS+', cout = (K+1)*N*L + (K-1)*L*(1+K/2);
                    case 'OCMP+', cout = 3*N*L + N^3/3 + 2*N*N*L + ...
                            N*L*L + 4*(K-1)*L*L + cout_inv(N,K);
                end
                
                if K > 1
                    cout = cout + N*K + Q*K* ...
                        (N + (K-1)*(3*N*L + N*N) + K*N + cout_inv(N,K));
                end
                cout = cout + cout_inv(N,K);
            else
                cout = nbre_mult;
            end
    end
    
    diff = cout - nbre_mult;
    if abs(diff) >= 1
        %        if algo == 'OCMP'
        %            fprintf(' %s %s %d \n', algo, version, diff)
        %        else
  %      fprintf(' %s %d \n', algo, diff)
        %        end
    end
end

function cout = cout_inv(N,k)
cout = k*(k*N + k^2/3 + N + k);


